@@ -26,7 +26,7 @@ def get_builder_restart(node, from_scratch=False, use_symlink=False):
     supported = (CpCalculation, NebCalculation, PhCalculation, PwCalculation)
 
     if node.process_class not in supported:
-        raise TypeError('calculation class `{}` of {} is not one of {}'.format(node.process_class, node, supported))
+        raise TypeError(f'calculation class `{node.process_class}` of {node} is not one of {supported}')
 
     builder = node.get_builder_restart()
 
